#! /usr/bin/env python
'''
Module Name:    TaskMgr.py

Description:     This is the main task manager for MUS.

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: Feb 12, 2015

'''
#importing core modules
import os
import sys
import subprocess
import time

#importing helper modules 

#from monitorupgrade import MUSConfigParser
from monitorupgrade import TLog as TLOG
from monitorupgrade import boot_update as bu
from monitorupgrade import Configs as Configs
from monitorupgrade import Utils as utils
from monitorupgrade import SampoVideoFwFlash as sampo
from monitorupgrade import EloTouchFwFlash as elo
from monitorupgrade import AtmelTouchFwFlash as atmel
from monitorupgrade import WistronVideoFwFlash as wistron


#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
UNKNOWN = -1
NO_MONITORS_ATTACHED = 0

#Error Codes
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_INVALID_PARAM = 21

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'


#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/mus/op_utils/sampo/4820MonUtil'
AFTER_LOCAL = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here    
ELO_DOWNLOAD = '/opt/toshiba/mus/op_utils/elo/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/mus/op_utils/elo/EloConfig.sh'
ELO_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/elo/recovery.txt'
ELO_INI = '/opt/toshiba/mus/UpdateTasks/TouchConfig/EloTouch/toshiba-mus2.1_up_2xx5xx_touch-config_elo/eloConfigOpt.ini'

#DWU
AIP_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/aipfwupdate/recovery.txt'
ATMEL_FW_RECOVERY_FILE = utils.ATMEL_FW_RECOVERY_FILE
ATMEL_XCFG_RECOVERY_FILE = utils.ATMEL_XCFG_RECOVERY_FILE
#gf->

DAWU_BASE_PATH='/opt/toshiba/mus/UpdateTasks/Touch/DWU'
FIRMWARE_DIR_NAME='fw'

#
# Internally there are now 2 types of Dawu 6149 Touch tasks
#
#    - Firmware update
#    - XCFG Load
#
ATMEL_FW_TASK='ATMEL_FW_TASK'
ATMEL_XCFG_TASK='ATMEL_XCFG_TASK'

#
# Used to verify that UP subdirectory adheres to the new 2.1 scheme
#
MON_6149_BASE_UP_FOLDER_NAME = 'toshiba-mus2.1_up_6149_touch'
#<-gf


#helpers
LSTMON = '/usr/bin/lstmon2'
RSTELO = '/usr/bin/rstelo2'
CHKELOPATCH = '/usr/bin/chk_elo_patch'
CHKFOUSB = '/usr/bin/chk_fousb'

#Hub vendors
CYPRESS_VID = '04b4'
SMC_VID = '0424'

#Product vendors
IBM_VID = '04b3'
ELO_VID = '04e7'
TGCS_VID = '0f66'
ATMEL_VID = '03eb'
UNKNOWN_VID = 'ffff'
UNKNOWN_PID = 'ffff'

PAD = ' '
SERIAL_LEN = 20

#Video Component ID;s
DWU_COMPONENTS = ['mcu',
                  'hw_cfg',
                  'scalar',
                  'capsense',
                  ]

UP_RPMS_DIR = '/opt/toshiba/mus/RpmUpdates'

# Manifest Update Types
UPDATE_TYPES = ['YAM_VIDEO_FW_UPDATE',   #0
                'ANK_VIDEO_FW_UPDATE',   #1
                'YAM_TOUCH_FW_UPDATE',   #2
                'ANK_TOUCH_FW_UPDATE',   #3
                'YNK_TOUCH_CONFIG',       #4
                'DWU_VIDEO_FW_UPDATE',   #5
                'DWU_TOUCH_FW_UPDATE',   #6
                'DWU_TOUCH_CONFIG',       #7
                '3MD_TOUCH_FW_UPDATE',   #8
                '3MD_TOUCH_CONFIG',       #9
                'SPECIAL_TASK'            #10   
                ]

#
# Individual task types for Dawu only
#
DWU_TOUCH_FW_UPDATE=UPDATE_TYPES[6]
DWU_TOUCH_CONFIG=UPDATE_TYPES[7]

# Verbose printed debug
ADBG=True
                
#======================================================== Main Task Manager Class ======================================================        


#==========================================================================================================
'''
TaskMgr - The main class to handle and call other helper classes for individual task
'''
#==========================================================================================================        
class TaskMgr:
    init = False
    fw_path=''
    fw_name=''
    l=''
    controller=''
    serial = ''
    hwid = ''
    retry=0
    RECOVERY_MODE = False
    HAS_4820_MON = False
    mon_list = []
    video_up_tasks = []
    touch_up_tasks = []
    pending_tasks = []
    rcv_cmd = ''
    
    def __init__(self):
        '''
        Constructor
        '''
        #print "TaskMgr Constructed"
        TLOG.logger.info('TaskMgr Constructed!')

#==========================================================================================================
# initialize() - function to check if task manager has already been initialized
#==========================================================================================================
    def initialize(self):
        
        if (self.init):
            #print'TaskMgr::initialize-Already Initialized'
            TLOG.logger.info('TaskMgr::initialize-Already Initialized')
        else:
            self.init = True
            #print'TaskMgr::initialize-Initialized OK'
            self.runRpmUpdates()
            TLOG.logger.info('TaskMgr::initialize-Initialized OK')
            

#==========================================================================================================
# checkFousb() - Check if elofousb KMP driver module is installed or not
#              - if installed then exit MUS 
#              - This is for MUS 1.0 only, MUS 1.1 don't need it
#==========================================================================================================
    def checkFousb(self):
        
        res = -1
        process = subprocess.Popen(CHKFOUSB, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            res = int ( line.strip('\n') )    
        process.wait()
        return int(res)


#==========================================================================================================
# checkEloUsbPatch() - Checks whether elousb driver with fw_quirk adjustment is installed or not
#                    - If not installed  then exit MUS
#                    - This is to ensure we don't have complications with 'broken firmware' workaround
#==========================================================================================================
    def checkEloUsbPatch(self):
        
        res = -1
        process = subprocess.Popen(CHKELOPATCH, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            res = int( line.strip('\n') )    
        process.wait()
        return res

#==========================================================================================================
# doRstElo()   - Reset EloUSB devices
#==========================================================================================================
    def doRstElo(self):
        
        #res = -1
        process = subprocess.Popen(RSTELO, shell=True, stdout=subprocess.PIPE)
        while True:
            line = process.stdout.readline()
            #print line
            if not line:
                break
            #TLOG.logger.info(line)
            #res = int ( line.strip('\n') )    
        process.wait()
    
        #return int(res)

#==========================================================================================================
# runRpmUpdates() - functions to check whether we have new UP rpm files in the rpmUpdate directory
#==========================================================================================================
    def runRpmUpdates(self):
        bu.process()
#==========================================================================================================
# getMonitorList() - function to get the list of all attached monitors and its details --Video/Touch FW, etc
#==========================================================================================================
    def getMonitorList(self):
        process = subprocess.Popen('lstmon2', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        while True:
            output_line = process.stdout.readline().strip('\n')
            if not output_line:
                break
            hub,entity,vid_pid,bcdDevice,controller = output_line.split(' ')
            self.mon_list.append((hub,entity,vid_pid,bcdDevice,controller ))                       
            
            process.wait()
        return len(self.mon_list)


 
#==========================================================================================================
#     def getMonitorList2_pre_atmel_elo_dawu_updates - function to get the list of all attached monitors and its details --Video/Touch FW, etc
#
#    
#    KEPT AS A REFERENCE
#    RENAMED FROM getMonitorList2
#    NOT USED ANY MORE AS THE INPUT FROM lstmon2.c HAS BEEN CHANGED TO ADD A HARDWARE ID FIELD
#==========================================================================================================
    def getMonitorList2_pre_atmel_elo_dawu_updates(self):
        process = subprocess.Popen('lstmon2 >> /tmp/monlist.txt', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait()                  
    
        #mon_list = []
        f = open('/tmp/monlist.txt', 'r+')
        lines = f.readlines()
        f.close() 
       
        process = subprocess.Popen('rm /tmp/monlist.txt ', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        process.wait() 
        
        curr_id = 0
        for line in lines:
            if ( "TDV" in line ):
                line = line.strip('\n')
                #hub_id, entity, vid_pid, bcdDevice, usb_dev_id,serial_1, serial_2, serial_3 = line.split(' ')
                tokens = line.split(' ')
                hub_id = tokens[0]
                entity = tokens[1]
                vid_pid = tokens[2]
                bcdDevice = tokens[3]
                usb_dev_id = tokens[4]
                serial =  tokens[5] 
                #print hub_id,entity,vid_pid,bcdDevice,controller1,controller2,controller3
                
            elif ( "ADT" in line ):
                line = line.strip('\n')
                #hub_id, entity, vid_pid, bcdDevice, usb_dev_id,serial_1, serial_2, serial_3 = line.split(' ')
                tokens = line.split(' ')
                hub_id = tokens[0]
                entity = tokens[1]
                vid_pid = tokens[2]
                bcdDevice = tokens[3]
                usb_dev_id = tokens[4]
                serial = tokens[5]

                #print hub_id,entity,vid_pid,bcdDevice,controller1,controller2,controller3
            else:
                line = line.strip('\n')
                hub_id, entity, vid_pid, bcdDevice, usb_dev_id, serial= line.split(' ')
                #print hub_id,entity,vid_pid,bcdDevice,controller
    
            if (hub_id[-1:] != curr_id):
     
                curr_id = hub_id[-1:]
              
                mon_ = TouchMonitor()
                mon_.setScreenNo(curr_id)
    
                if entity == 'TDV' or entity == 'SYV' or entity == 'SAV':
                   
                    mon_.setVideoFwLevel(bcdDevice)
                    mon_.setVideoSerialNumber(serial)
                    mon_.setVideoVidPid(vid_pid)
                    mon_.setVideoDevId(usb_dev_id)
                    
                    if entity == 'TDV':
                        mon_.setMonType('DWU')
                    elif entity == 'SYV':
                        mon_.setMonType('YAM')
                        self.HAS_4820_MON = True
                    elif entity == 'SAV':
                        mon_.setMonType('ANK')
                        self.HAS_4820_MON = True
                
                elif entity =='ADT' or entity == 'EKT' or entity == 'AIT':
                    
                    l = len(serial)
                    print 'serial_len:' + str(l)
                    if ( entity =='ADT' or entity == 'AIT') :
                        if l < SERIAL_LEN:
                            pad_count = SERIAL_LEN - l
                            for i in range(pad_count):
                                serial += PAD
                                
                                 
                    mon_.setTouchSerialNumber(serial)
                    mon_.setTouchFwLevel(bcdDevice)
                    mon_.setTouchVidPid(vid_pid)
                    mon_.setTouchDevId(usb_dev_id)
                    
                    #print entity,vid_pid,bcdDevice,serial
               
            elif (hub_id[-1:] == curr_id):
            
                if entity == 'TDV' or entity == 'SYV' or entity == 'SAV':
               
                    mon_.setVideoFwLevel(bcdDevice)
                    mon_.setVideoSerialNumber(serial)
                    mon_.setVideoVidPid(vid_pid)
                    mon_.setVideoDevId(usb_dev_id)
                    
                    if entity == 'TDV':
                        mon_.setMonType('DWU')
                    elif entity == 'SYV':
                        mon_.setMonType('YAM')
                        self.HAS_4820_MON = True
                    elif entity == 'SAV':
                        mon_.setMonType('ANK')
                        self.HAS_4820_MON = True
    
                elif entity =='ADT' or entity == 'EKT':
                    
                    l = len(serial)
                    #print 'serial_len:' + str(l)
                    if ( entity =='ADT' ) :
                        if l < SERIAL_LEN:
                            pad_count = SERIAL_LEN - l
                            for i in range(pad_count):
                                serial += PAD
                    
                    mon_.setTouchFwLevel(bcdDevice) 
                    mon_.setTouchSerialNumber(serial)
                    mon_.setTouchVidPid(vid_pid)
                    mon_.setTouchDevId(usb_dev_id)
                    
            
                self.mon_list.append(mon_)
        
        print '#_of_monitors attached: ' + str(len(self.mon_list))
        return len(self.mon_list)

#==========================================================================================================
#     def getMonitorList2 - function to get the list of all attached monitors and its details --Video/Touch FW, etc
#    
#    INPUT FROM lstmon2.c HAS BEEN CHANGED TO ADD A HARDWARE ID FIELD
#==========================================================================================================
    def getMonitorList2(self):
        ''' getMonitorList2 - function to get the list of all attached monitors and its details --Video/Touch FW, etc.
      '''
        # reset this->mon_list
        self.mon_list = []
 
        TLOG.logger.info('TaskMgr::getMonitorList2: ')
 
        # Utility function 'utils.getMonList()' does the main work of getting and parsing the output of /usr/bin/lstmon2
        #
        # tmp_monlist is an array of controller chip descriptors. Each descriptor is a dict.
        tmp_monlist = utils.getMonList()
            
        # temporary local object containing TouchMonitor objects
        tmon = {}
        
        for tm in tmp_monlist:
            
            hub_id = tm[utils.MON_HUB_ID]
            
            # Make new TouchMonitor object if needed
            if not (tmon.has_key(hub_id)):
                touch_mon = TouchMonitor()
                hub = hub_id[-1:]
                touch_mon.setScreenNo(hub)
                tmon[hub_id] = touch_mon
                print tmon

            # From here on refer to TouchMonitor by it's hub_id Key

            # Video devices
            if tm[utils.MON_ENTITY] == 'TDV' or tm[utils.MON_ENTITY] == 'SYV' or tm[utils.MON_ENTITY] == 'SAV':
                   
                tmon[hub_id].setVideoFwLevel(tm[utils.MON_BCDDEVICE])
                tmon[hub_id].setVideoSerialNumber(tm[utils.MON_SERIAL])
                tmon[hub_id].setVideoVidPid(tm[utils.MON_VIDPID])
                tmon[hub_id].setVideoDevId(tm[utils.MON_USB_DEVID])
                tmon[hub_id].setVideoHardwareId(tm[utils.MON_HWID])
                    
                if tm[utils.MON_ENTITY] == 'TDV':
                    tmon[hub_id].setMonType('DWU')
                
                elif tm[utils.MON_ENTITY] == 'SYV':
                    tmon[hub_id].setMonType('YAM')
                    self.HAS_4820_MON = True
            
                elif tm[utils.MON_ENTITY] == 'SAV':
                    tmon[hub_id].setMonType('ANK')
                    self.HAS_4820_MON = True
                
            # Touch devices
            elif tm[utils.MON_ENTITY] =='ADT' or tm[utils.MON_ENTITY] == 'EKT' or tm[utils.MON_ENTITY] == 'AIT':

                tmon[hub_id].setTouchSerialNumber(tm[utils.MON_SERIAL])
                tmon[hub_id].setTouchFwLevel(tm[utils.MON_BCDDEVICE])
                tmon[hub_id].setTouchVidPid(tm[utils.MON_VIDPID])
                tmon[hub_id].setTouchDevId(tm[utils.MON_USB_DEVID])
                tmon[hub_id].setTouchHardwareId(tm[utils.MON_HWID])
                
            # Video devices
            elif tm[utils.MON_ENTITY] == 'ADD':
              
                TLOG.logger.error('TaskMgr::getMonitorList2 - Not in recovery mode but we do have an Atmel controller in DFU mode')
                
                '''  Note we can 'potentially' add a special task to recover this assuming we have enough info to risk applying a task
                Unlikely though! We just don't have enough info back from a DFU Atmel controller. We don't know the serial number
                and we don't know the Hardware Id so we might not apply the correct FW.
            '''
                tmon[hub_id].setTouchSerialNumber(tm[utils.MON_SERIAL])      # dummy 'DFU' from lstmon2
                tmon[hub_id].setTouchFwLevel(tm[utils.MON_BCDDEVICE])       # version of bootloader
                tmon[hub_id].setTouchVidPid(tm[utils.MON_VIDPID])                     # 03ec:211D
                tmon[hub_id].setTouchDevId(tm[utils.MON_USB_DEVID])             # valid host-usbdevice
                tmon[hub_id].setTouchHardwareId(tm[utils.MON_HWID])               # dummy NOHWID from lstmon2
            
            else:
                
                TLOG.logger.error('TaskMgr::getMonitorList2 - Unknown Entity type: ' + tm[utils.MON_ENTITY])
                raise IOError('TaskMgr::getMonitorList2 - lstmon2 - Unknown Chip/Entity: ' + tm[utils.MON_ENTITY])
        #---#
            
        # use temp object to populate utils.MON_list    
        for k, v in tmon.iteritems():
            print k, '->', v
            self.mon_list.append(v)
                
        mon_list_len = len(self.mon_list)
        
        print '#_of_monitors attached: ' + str(mon_list_len)
        return mon_list_len
           

#==========================================================================================================
# parseFirmwareLevel() - returns the firmware level embedded on each update folders
#
# This will not work for new style 2.1 Dawu Touch UPs as there is now a GUID and Not an embedded FW level
#
# So thank goodness there is a manifest ;) 
#==========================================================================================================   
    def parseFirmwareLevel(self,rpm):
    
        level_str =''
        parts = rpm.split("_")
        num_parts = len(parts)
        if (num_parts>0):
            pattern = parts[0]
        if (num_parts>1):
            mus_rpm_type = parts[1]
        if (num_parts>2):
            target_models = parts[2]
        if (num_parts>3):
            update_type = parts[3]
        if (num_parts>4):
            tmp = parts[4]
            endparts = tmp.split("-")
            num_endparts = len(endparts)
            if (num_endparts>0):
                level_str = endparts[0]
        #print ' FW LEVEL:', level_str
        TLOG.logger.info('TaskMgr::parseFirmwareLevel() - FW LEVEL: ' + level_str)
                
        count = len(level_str)
        fw_level = level_str[count-2:]    
        return fw_level

#==========================================================================================================
# checkForTouchFwUpdate() - check whether we have necessary Touch Firmware Update
#                         - if there are then it will update the pending tasks list
#==========================================================================================================    
    def  checkForTouchFwUpdate(self,vid_pid,fw,controller_serial,mon_type,devid, hwid=None):
        
	# vid and pid will be empty if monitor is spoiled
	if vid_pid: 
            vid,pid = vid_pid.split(':') 
        else:
	    vid = UNKNOWN_VID
            pid = UNKNOWN_PID
	
        # Yanak: Elo Touch
        if vid == ELO_VID:
            print '[Touch Firmware - ELO]'
            #
            # Last 2 digits
            #
            elo_touch_fw = fw[2:]
            #
            # Skip first 2 digits
            #
            elo_serial = controller_serial[2:]
            TLOG.logger.info('TaskMgr::checkForVideoFwUpdate() -[Touch Firmware]')
            if (pid == "0009" or pid == "0030"):
            #Check current to see if this is the Touch for the Current Monitor
                if (mon_type == 'YAM'):
                    self.updatePendingTaskList('EloTouchFwFlashTask',\
                                               elo_touch_fw,elo_serial,'/opt/toshiba/mus/UpdateTasks/Touch/YAM')
                elif (mon_type == 'ANK'):
                    self.updatePendingTaskList('EloTouchFwFlashTask',\
                                               elo_touch_fw,elo_serial,'/opt/toshiba/mus/UpdateTasks/Touch/ANK')
                    
        # Dawu: Atmel Touch 
        elif vid  == ATMEL_VID: 
                    print '[Touch Firmware - Atmel]'
                    TLOG.logger.info('TaskMgr::checkForTouchFwUpdate() -[Touch Firmware]')
                    #
                    # All 4 digits
                    #
                    atmel_touch_fw = fw
                    #
                    # Use all
                    #
                    atmel_serial = controller_serial
                    base_fw_path = DAWU_BASE_PATH
                    # Check chip fw and hwid against any installed dawu touch tasks
                    if os.path.exists(base_fw_path):                 
                        self.updatePendingDWUTouchTasks(base_fw_path, atmel_touch_fw, devid, atmel_serial, hwid)                    
                                
                
    
#==========================================================================================================
# checkForVideoFwUpdate() - check whether we have necessary Video Firmware Update
#                         - if there are then it will update the pending tasks list
#==========================================================================================================    
    def  checkForVideoFwUpdate(self,vid_pid,fw,controller,mon_type,devid):
        
        # vid and pid will be empty if monitor is spoiled
        if vid_pid:
            vid,pid = vid_pid.split(':') 
        else:
            vid = UNKNOWN_VID
            pid = UNKNOWN_PID
        
        #YANAK
        if (vid == IBM_VID):
            video_fw = fw[2:]
            
            print '[Video Firmware]'
            TLOG.logger.info('TaskMgr::checkForVideoFwUpdate() -[Video Firmware]')
            serial,model = controller.split(';')
            serial = serial[2:]
            if (mon_type == 'YAM'):  # Found Yam Monitor
                self.updatePendingTaskList('SampoVideoFwFlashTask',video_fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/YAM')
            elif (mon_type == 'ANK'):  # Found Ank Monitor
                self.updatePendingTaskList('SampoVideoFwFlashTask',video_fw,serial,'/opt/toshiba/mus/UpdateTasks/Video/ANK')
        #DAWU             
        elif (vid == TGCS_VID):    
            print '[Video Firmware]'
            TLOG.logger.info('TaskMgr::checkForVideoFwUpdate() -[Video Firmware]')
            
            serial = controller
            fws = fw.split(',')
            for idx ,comp_fw in enumerate(fws) :
                print 'FW_Component_'+ str (idx)
                comp_fw = comp_fw[2:]
                fw_path = '/opt/toshiba/mus/UpdateTasks/Video/DWU/'
                #fw_path = '/opt/toshiba/mus/UpdateTasks/Video/DWU/'+ DWU_COMPONENTS[idx]
                if os.path.exists(fw_path):
                    self.updatePendingDWUVideoTasks('WistronVideoFwFlashTask',comp_fw,serial,fw_path,DWU_COMPONENTS[idx])


#==========================================================================================================
# updatePendingDWUTouchTasks_pre_atmel_elo_dawu_updates() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
#    KEPT AS A REFERENCE
#    RENAMED FROM updatePendingDWUTouchTasks
#    NOT USED ANY MORE AS THERE IS NOW A GUID AND NOT AN EMBEDDED FW VERSION IN THE FOLDER NAME
#==========================================================================================================
    def updatePendingDWUTouchTasks_pre_atmel_elo_dawu_updates(self, task_name, fw, path, devid, serial, hwid):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        name = ''
        
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            #for firmware levels with bcd values, we convert them to hex
            fw = int(fw,16)
        
        # keep a record of old firmware value
        current_fw = int(fw)            
        
        max_level = 0
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames:   
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            
            # hold the string representation of the folder name
            folder_str = folder               
                         
            #compare the firmware of the device to the folder name 
            #corresponding to available firmware updates            
            if int(folder) > max_level:
                max_level = int(folder)
                
                #assign the new folder location
                new_fw_str = folder_str       
                name = folder_name
                print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - Found! FW package name:' + new_fw_str)
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - Found! FW package name:' + name)

        fw_path = (os.path.join(dirpath,name))
        
        tskcfg = Configs.UpdateTaskConfig(fw_path)
        fw = tskcfg.getFirmwareLevel()
        fw = fw[2:]
        
        #gf->
        ###xcfg = tskcfg.getXcfgLevel()
    
        match_hwid = tskcfg.getMatchingHardwareId()
        if (match_hwid is None):
            print 'No MATCH_HWID found in tsk manifest'
            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTasks() - No MATCH_HWID found in tsk manifest')
            return

        print 'MATCH_HWID:', match_hwid
        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - MATCH_HWID: ' + match_hwid)
        
        if not (hwid == match_hwid):
            print 'Hardware Id', hwid, 'does not equal MATCH_HWID:', match_hwid
            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTasks() - Actual Hardware Id ' \
                               + hwid + ' does not equal manifest MATCH_HWID: ' + match_hwid)
            return
        #<-gf
        
        print int(fw)
        if ( int(fw) > current_fw) :
            self.touch_up_tasks.append(  (task_name, fw_path, devid, serial, hwid)  )
        
        #Load only configuration. This is if there are same FW level but different configuration    
        elif ( int(fw) == current_fw) :
            task_name = 'AtmelTouchXcfgLoadTask'
            self.touch_up_tasks.append(  (task_name, fw_path, devid, serial, hwid)  ) 
        
        else:
            print 'No Firmware/XCFG Update Available!'  
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTasks() - Firmware/XCFG Update Available!')

                                                             
#===============================================================================
# #==========================================================================================================
# # updatePendingDWUTouchTasks() - update the PendingTasks list based on the firmware on the chip
# #==========================================================================================================    
#    #TODO: updatePendingDWUTouchTasks_gw
#    def updatePendingDWUTouchTasks_gw(self, task_base_path, chip_fw,  chip_devid, chip_serial, chip_hwid):
#        if ADBG: print '--------'
#        if ADBG: print 'CHIP'
#        if ADBG: print '--------'
#        if ADBG: print task_base_path
#        if ADBG: print chip_fw
#        if ADBG: print chip_devid
#        if ADBG: print chip_serial
#        if ADBG: print chip_hwid
#        if ADBG: print '--------'
#        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- CHIP - '\
#                         +' task_base_path='+task_base_path\
#                         +' chip_fw='+chip_fw\
#                         +' chip_devid='+chip_devid\
#                         +' chip_serial='+chip_serial\
#                         +' chip_hwid='+chip_hwid)
#        
#        #
#        # Keep a record of chip firmware value NNNN as an integer. Assumes this is a hex string of form like '1028'
#        #
#        chip_fwi = int(chip_fw,16)
#    
#        #
#        # Get a list of all the directories inside the opt/toshiba/mus/UpdateTask/Touch/DWU [or future DAZ] directory
#        #
#        for (base_dirpath,base_dirnames,base_filenames) in os.walk(task_base_path):
#            # Yield block
#            break
#        
#        #
#        # List of candidate touch update tasks for this chip
#        #
#        potential_tasks = []
#        
#        #
#        # Iterate through the list of firmware directories; each one is a potential Dawu MUS UP PKG for this chip
#        #
#        for up_base_dir in base_dirnames:
#            if up_base_dir.startswith(MON_6149_BASE_UP_FOLDER_NAME):
#                up_base_path = (os.path.join(base_dirpath, up_base_dir))
#                print
#                if ADBG: print 'UP BASE:', up_base_path
#                if ADBG: print '========'
#                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- UP BASE - '+up_base_path)
#                # get a list of all the first-level subdirctories inside the UP
#                for (up_dirpath,up_dirnames,up_filenames) in os.walk(up_base_path):
#                    #Yield block
#                    break
#                #
#                # find HWID subdirectory associated with chip HWID
#                #
#                found_hwid = False
#                subtask_path = ''
#                for hwid_dir in up_dirnames:
#                    if ADBG: print hwid_dir
#                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Looking at hardware directory - '+hwid_dir)
#                    # Change any underscores in HWID directory names to dashes
#                    # MUS UPs will have only '-' dashes but 4690/Diags may use '_'
#                    test_hwid_dir = hwid_dir.replace('_', '-')
#                    if test_hwid_dir==chip_hwid:
#                        subtask_path = (os.path.join(up_base_path, hwid_dir))
#                        found_hwid = True
#                        break
#                #
#                # If associated HWIDs found add to list of potential candidates
#                #
#                if found_hwid:
#                    if ADBG: print 'found subdirectory: ', hwid_dir, 'for chip HWID:', chip_hwid
#                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Found subdirectory - '\
#                                     +hwid_dir+' for chip HWID '+chip_hwid)
#                    # 'tsk.manifest' will be appended by UpdateTaskConfig class __init__
#                    manifest_path = subtask_path
#                    cfg = Configs.UpdateTaskConfig(manifest_path)
#                    tsktype  =  cfg.getTaskType()
#                    manifest_total_fw_str  =  cfg.getFirmwareLevel()
#                    # This has enclosing quotes from the manifest
#                    manifest_fw = manifest_total_fw_str.strip('\'')
#                    if ADBG: print 'manifest firmware: ' + manifest_fw
#                    if ADBG: print tsktype
#                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Manifest firmware - '+manifest_fw)
#                    manifest_fwi = 0
#                    if len(manifest_fw) != 4:
#                        # should be something like '1033'
#                        if ADBG: print 'Warning: manifest firmware string wrong length: ' + str(len(manifest_fw)) + ' .. ignoring task'
#                        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Warning: manifest firmware string wrong length: '\
#                                          + str(len(manifest_fw)) + ' .. ignoring task')
#                    else:
#                        try:
#                            # Convert firmware to an integer value
#                            manifest_fwi = int(manifest_fw,16)
#                            # Add candidate task to the list of potential update tasks for this chip 
#                            pot_task = {'fwi' : manifest_fwi, 'stp' : subtask_path}
#                            potential_tasks.append(pot_task)
#                        except ValueError:
#                            if ADBG: print 'Error: non-integer manifest FW level', manifest_fw
#                            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask- Error: non-integer manifest FW level - '+manifest_fw)
#                          
#        #
#        # Find the task with the highest FW level.  Assumes if duplicate task FW levels then the last one 'wins' 
#        #
#        print
#        highest_task_fwi = 0
#        highest_task = None
#        for task in potential_tasks:
#            if ADBG: print 'chip:', hex(chip_fwi)
#            task_fwi = task['fwi']
#            if ADBG: print 'task:', hex(task_fwi)
#            if ADBG: print task['stp']
#            #
#            if (task_fwi >= highest_task_fwi):
#                highest_task_fwi = task_fwi
#                highest_task = task
#    
#        #
#        # Add the task to the touch tasks queue if FW in the package is >= chip FW level 
#        #
#        if not highest_task is None:
#            #
#            task_path = highest_task['stp']
#            #
#            if ADBG: print 'Highest Task FW Level has FW Version:', hex(highest_task_fwi), ' and path', task_path
#            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Highest Task FW Level has FW Version: '+ str(hex(highest_task_fwi)) + ' and path: '+task_path)
#            #
#            if highest_task_fwi > chip_fwi:
#                if ADBG: print 'Chip FW at', hex(chip_fwi), 'will be updated to ', hex(highest_task_fwi), 'and will have an XCFG update'
#                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip FW at: '\
#                                 +str(hex(chip_fwi))+' will be updated to: '\
#                                 +str(hex(highest_task_fwi)) + ' and will have an XCFG update')
#                # Task name is AtmelTouchFwFlashTask
#                self.touch_up_tasks.append(  (ATMEL_FW_AND_XCFG_TASK, task_path, chip_devid, chip_serial, chip_hwid)  )
#            elif highest_task_fwi == chip_fwi:
#                if ADBG: print 'Chip FW at', hex(chip_fwi), 'has the same firmware level as ', hex(highest_task_fwi), 'and will only have an XCFG update'
#                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip FW at: '\
#                                 +str(hex(chip_fwi))+' has the same firmware level as manifest update: '\
#                                 +str(hex(highest_task_fwi)) + ' and will only have an XCFG update')
#                # Task name is AtmelTouchXcfgLoadTask
#                self.touch_up_tasks.append(  (ATMEL_XCFG_LOAD_ONLY_TASK, task_path, chip_devid, chip_serial, chip_hwid)  )
#            else:
#                if ADBG: print 'Chip FW at', hex(chip_fwi), 'is up to date. No task to apply'
#                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip FW at: '+ str(hex(chip_fwi)) + ' is up to date. No task to apply')
#        else:
#            if ADBG: print 'No available update tasks for this chip'
#            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- No available update tasks for this chip')
#===============================================================================

#==========================================================================================================
# updatePendingDWUTouchTasks() - update the PendingTasks list based on the firmware on the chip
#==========================================================================================================    
    #TODO: updatePendingDWUTouchTasks
    def updatePendingDWUTouchTasks(self, task_base_path, chip_fw,  chip_devid, chip_serial, chip_hwid):
        if ADBG: print '--------'
        if ADBG: print 'CHIP'
        if ADBG: print '--------'
        if ADBG: print chip_fw
        if ADBG: print chip_devid
        if ADBG: print chip_serial
        if ADBG: print chip_hwid
        if ADBG: print '--------'
        
        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- CHIP - '\
                        +' chip_fw='+chip_fw\
                        +' chip_devid='+chip_devid\
                        +' chip_serial='+chip_serial\
                        +' chip_hwid='+chip_hwid)
        
        #
        # Check Dawu touch task directory
        #
        if not os.path.exists(task_base_path):
            if ADBG: print 'Cannot access touch task directory ', task_base_path
            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask- Cannot access touch task directory '+task_base_path)
            return
        
        #
        # Check Dawu touch task directory has subdirectories (UPs)
        #
        try:
            os.listdir(task_base_path)
        except:
            if ADBG: print 'No directories (UPs) in  ', task_base_path
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- No directories (UPs) in '+task_base_path)
            return
        
        #
        # Keep a record of chip firmware version FFFF as an integer. Assumes this is a hex string of form like '1028'
        # Also keep a record of chip XCFG version XX as an integer. Assumes this is a decimal string of form like '23'
        # The incoming chip_fw string is of the form: 'FFFF,XX'
        #
        try:
            fw, xc = chip_fw.split(',')
        except ValueError:
            if ADBG: print 'Malformed chip FW string ', chip_fw
            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask - Malformed chip FW string '+chip_fw)
            return
        try:
            chip_fw_i = int(fw,16)
        except ValueError:
            if ADBG: print 'Invalid Chip firmware value (NaN) ', fw
            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask - Invalid Chip firmware value (NaN) '+fw)
            return
        try:
            chip_xcfg_i = int(xc,10)
        except ValueError:
            if ADBG: print 'Invalid Chip XCFG value (NaN) ', xc
            TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask - Invalid Chip firmware value (NaN) '+xc)
            return
    
        #
        # Get a list of all the directories inside the opt/toshiba/mus/UpdateTask/Touch/DWU [or future DAZ] directory
        #
        for (base_dirpath,base_dirnames,base_filenames) in os.walk(task_base_path):
            # Yield block
            break
        
        #
        # List of candidate touch update tasks for this chip
        #
        potential_fw_tasks = []
        potential_xcfg_tasks = []
        
        #
        # Iterate through the list of firmware directories; each one may have a potential Dawu MUS UP PKG for this chip
        #
        for up_base_dir in base_dirnames:
            if up_base_dir.startswith(MON_6149_BASE_UP_FOLDER_NAME):
                up_base_path = (os.path.join(base_dirpath, up_base_dir))
                if ADBG: print
                if ADBG: print '========'
                if ADBG: print 'UP BASE:', up_base_path
                if ADBG: print '========'
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- UP BASE - '+up_base_path)
    
                #
                # Get a list of all the first-level subdirctories inside the UP
                #
                for (up_dirpath,up_dirnames,up_filenames) in os.walk(up_base_path):
                    #Yield block
                    break
                
                #
                # Find HWID subdirectory associated with chip HWID
                #
                found_hwid = False
                subtask_path = ''
                for hwid_dir in up_dirnames:
                    if ADBG: print 'Looking at hardware directory - ', hwid_dir
                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Looking at hardware directory - '+hwid_dir)
                    # Change any underscores in HWID directory names to dashes
                    # MUS UPs will have only '-' dashes but 4690/Diags may use '_'
                    test_hwid_dir = hwid_dir.replace('_', '-')
                    if test_hwid_dir==chip_hwid:
                        subtask_path = (os.path.join(up_base_path, hwid_dir))
                        found_hwid = True
                        break
                    
                #
                # If associated HWIDs found try to add to the list of potential candidates
                #
                if found_hwid:
                    #
                    if ADBG: print 'Found HWID subdirectory: ', hwid_dir, 'for chip HWID:', chip_hwid
                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Found subdirectory - '\
                                     +hwid_dir+' for chip HWID '+chip_hwid)
                    
                    #
                    # Has this task had too many reboots?
                    #
                    reboot_count = utils.getRebootCount(subtask_path)
                    if reboot_count is None:
                        if ADBG: print 'Error getting reboot count - ignoring UP'
                        TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask- Error getting reboot count - ignoring UP')
                        continue
                    if ADBG: print 'UP reboot count: ', reboot_count
                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- UP reboot count: '+str(reboot_count))
                    #TODO: MAX_ATMEL_TOUCH_REBOOT_CNT
                    if reboot_count > 2:
                        if ADBG: print 'This task has had too many failed recovery reboots - ignoring UP'
                        TLOG.logger.warn('TaskMgr::updatePendingDWUTouchTask- This task has had too many failed recovery reboots - ignoring UP')
                        continue                       
                    
                    #
                    # Get manifest information
                    #
                    manifest_path = subtask_path
                    if not os.path.exists(manifest_path):
                        if ADBG: print 'no manifest file found'
                        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- No manifest file found - ignoring UP')
                        continue
                    # 'tsk.manifest' will be appended by UpdateTaskConfig class __init__
                    cfg = Configs.UpdateTaskConfig(manifest_path)
                    try:
                        tsktype  =  cfg.getTaskType()
                        manifest_total_fw_str  =  cfg.getFirmwareLevel()
                        manifest_total_xcfg_str = cfg.getXcfgLevel()
                    except:
                        if ADBG: print 'Bad/missing manifest TYPE, FW_LEVEL and/or XCFG_LEVEL'
                        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Bad/missing manifest TYPE, FW_LEVEL and/or XCFG_LEVEL - ignoring UP')
                        continue
                    #
                    # These have enclosing quotes from the manifest
                    manifest_fw = manifest_total_fw_str.strip('\'')
                    manifest_xcfg = manifest_total_xcfg_str.strip('\'')
                    # should be something like '2.3' .. concatenate the digits
                    manifest_xcfg = manifest_xcfg.replace('.', '')
                    if ADBG: print 'manifest firmware: ' + manifest_fw
                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Manifest FW - '+manifest_fw)
                    if ADBG: print 'manifest xcfg: ' + manifest_xcfg
                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Manifest XCFG - '+manifest_xcfg)
                    if ADBG: print tsktype
                    TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Task type - '+tsktype)
                    manifest_fw_i = 0
                    if len(manifest_fw) != 4:
                        # should be something like '1033'
                        if ADBG: print 'Warning: manifest firmware string wrong length: ' + str(len(manifest_fw)) + ' .. ignoring UP'
                        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Warning: manifest firmware string wrong length: '\
                                          + str(len(manifest_fw)) + ' .. ignoring task')
                        continue
                    #
                    elif len(manifest_xcfg) != 2:
                        # should be something like '23'
                        if ADBG: print 'Warning: manifest xcfg string wrong length: ' + str(len(manifest_xcfg)) + ' .. ignoring UP'
                        TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Warning: manifest xcfg string wrong length: '\
                                          + str(len(manifest_xcfg)) + ' .. ignoring task')
                        continue
                    #
                    try:
                        #
                        # Convert firmware version to an integer value - base 16
                        #
                        manifest_fw_i = int(manifest_fw,16)
                        #
                    except ValueError:
                        if ADBG: print 'Error: non-integer manifest FW level', manifest_fw
                        TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask- Error: non-integer manifest FW level - '+manifest_fw)
                        continue
                    try:
                        #
                        # Convert xcfg to an integer value - base 10
                        #
                        manifest_xcfg_i = int(manifest_xcfg,10)
                        #
                    except ValueError:
                        if ADBG: print 'Error: non-numeric manifest XCFG level', manifest_xcfg
                        TLOG.logger.error('TaskMgr::updatePendingDWUTouchTask- Error: non-numeric manifest XCFG level - '+manifest_xcfg)
                        continue
                        
                    #
                    # Add candidate task to the appropriate list of potential update tasks for this chip - there are 2 list types:
                    #    - FW Updates
                    #    - XCFG Updates
                    #
                    # Note: There is no such thing as a FW Update only task without an XCFG. We control that as we make the UPs
                    #
                    pot_task = {'fw_i' : manifest_fw_i,  'xcfg_i' : manifest_xcfg_i,  'stp' : subtask_path}
                    if tsktype == DWU_TOUCH_FW_UPDATE:
                        #
                        # Add to list of possible FW tasks
                        #
                        potential_fw_tasks.append(pot_task)
                    #
                    # All tasks have an XCFG so always add task to this list
                    #
                    potential_xcfg_tasks.append(pot_task)
        
        ### ENDFOR - for up_base_dir in base_dirnames: ###
          
        if len(potential_fw_tasks) is 0 and len(potential_xcfg_tasks) is 0:
            if ADBG: print 'TaskMgr::updatePendingDWUTouchTask- No potential candidate Atmel Touch tasks found - exiting'
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- No potential candidate Atmel Touch tasks found - exiting')
            #
            return
    
        
        #############
        # FW TASKS #
        #############
                          
        if not len(potential_fw_tasks) is 0:
            #
            # If we get here there must be at least 1 FW task in the list
            #
            # Find the potential task candidate with the highest FW level.  
            #     - Since they are the same FW update inside - then we let the last one encountered 'win'
            #
            highest_fw_task = None
            highest_task_fw_i = 0
            #
            for fwtask in potential_fw_tasks:
                fwtask_fw_i = fwtask['fw_i']
                fwtask_path = fwtask['stp']
                if ADBG: print 'Looking at FW + XCFG task: ',fwtask_path, ' for FW level: ', hex(fwtask_fw_i),' - The Chip level is ', chip_fw
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Looking at FW + XCFG task:'+fwtask_path\
                                 +' for FW level '+str(hex(fwtask_fw_i))\
                                 +' - The Chip level is '+chip_fw)
                #
                # If this is the currently highest FW or same as current highest FW then temporarily store this one
                #
                if (fwtask_fw_i >= highest_task_fw_i):
                    highest_task_fw_i = fwtask_fw_i
                    highest_fw_task = fwtask
            
            #
            # Add the highest  task to the touch tasks queue if FW in the package is > chip FW level 
            #
            highest_fw_task_path = highest_fw_task['stp']
            highest_task_fw_i = highest_fw_task['fw_i']
            #
            if ADBG: print 'Highest Task FW Level is:', hex(highest_task_fw_i),' and path', highest_fw_task_path
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Highest Task FW Level is'\
                             + str(hex(highest_task_fw_i))\
                             +' and path: '+highest_fw_task_path)
            #
            if highest_task_fw_i > chip_fw_i:
                #
                # ADD FW TASK
                #
                if ADBG: print 'Chip FW at', hex(chip_fw_i), 'will be updated to ', hex(highest_task_fw_i)
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip FW at: '\
                                 +str(hex(chip_fw_i))+' will be updated to: '\
                                 +str(hex(highest_task_fw_i)))
                self.touch_up_tasks.append(  (ATMEL_FW_TASK, highest_fw_task_path, chip_devid, chip_serial, chip_hwid)  )
            else:
                if ADBG: print 'Chip FW at', hex(chip_fw_i), 'is up to date. No FW task to apply'
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip FW at: '+ str(hex(chip_fw_i)) + ' is up to date. No FW task to apply')
        else:
            if ADBG: print 'No available FW update tasks for this chip'
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- No available FW update tasks for this chip')
      
      
        ###############
        # XCFG TASKS #
        ###############
         
        #                      
        # Now we have 0 .. n candidate XCFG tasks to consider from all of the tasks for the HWID
        #   
        if not len(potential_xcfg_tasks) is 0:
            #
            # If we get here there must be at least 1 XCFG task in the list
            #
            # Find the potential task candidate with the highest XCFG level.  
            #     - Since they are the same XCFG update inside - then we let the last one encountered 'win'
            #
            highest_xcfg_task = None
            highest_task_xcfg_i = 0
            #
            for xcfgtask in potential_xcfg_tasks:
                xcfgtask_xcfg_i = xcfgtask['xcfg_i']
                xcfgtask_path = xcfgtask['stp']
                if ADBG: print 'Looking at task: ',xcfgtask_path, ' for XCFG level: ', xcfgtask_xcfg_i ,' - The Chip level is ', chip_fw
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Looking at task:'+xcfgtask_path\
                                 +' for XCFG level '+str(xcfgtask_xcfg_i )\
                                 +' - The Chip level is '+chip_fw)
                #
                # If this is the currently highest XCFG or same as current highest XCFG then temporarily store this one
                #
                if (xcfgtask_xcfg_i  >= highest_task_xcfg_i):
                    highest_task_xcfg_i = xcfgtask_xcfg_i 
                    highest_xcfg_task = xcfgtask
    
            #
            # Add the highest  task to the touch tasks queue if XCFG in the package is > chip XCFG level 
            #
            highest_xcfg_task_path = highest_xcfg_task['stp']
            highest_task_xcfg_i = highest_xcfg_task['xcfg_i']
    
            if highest_task_xcfg_i > chip_xcfg_i:
                #
                # ADD XCFG TASK
                #
                if ADBG: print 'Chip XCFG at', chip_xcfg_i, 'will be updated to ', highest_task_xcfg_i
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip XCFG at: '\
                                 +str(chip_xcfg_i)+' will be updated to: '\
                                 +str(highest_task_xcfg_i))
                self.touch_up_tasks.append(  (ATMEL_XCFG_TASK, highest_xcfg_task_path, chip_devid, chip_serial, chip_hwid)  )
            else:
                if ADBG: print 'Chip XCFG at', chip_xcfg_i, 'is up to date. No XCFG task to apply'
                TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- Chip XCFG at: '+ str(chip_xcfg_i) + ' is up to date. No XCFG task to apply')
            
        else:
            if ADBG: print 'No available XCFG update tasks for this chip'
            TLOG.logger.info('TaskMgr::updatePendingDWUTouchTask- No available XCFG update tasks for this chip')

        
                                                             
#==========================================================================================================
# updatePendingDWUVideoTasks() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
    
    def updatePendingDWUVideoTasks(self,task_name,fw,serial,path,comp):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        name = ''
        
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            #for firmware levels with bcd values, we convert them to hex
            fw = int(fw,16)         
        
        # keep a record of old firmware value
        current_fw = int(fw)            
        
        max_level = 0
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames:   
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            
            # hold the string representation of the folder name
            folder_str = folder               
                         
            #compare the firmware of the device to the folder name 
            #corresponding to available firmware updates            
            if int(folder) > max_level:
                max_level = int(folder)
                
                #assign the new folder location
                new_fw_str = folder_str       
                name = folder_name
                #print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingDWUVideoTasks() - Found! FW package name:' + new_fw_str)
                TLOG.logger.info('TaskMgr::updatePendingDWUVideoTasks() - Found! FW package name:' + name)

        fw_path = (os.path.join(dirpath,name))
        
        controller = comp
        
        try:
            if (comp == DWU_COMPONENTS[0]) :
                fw_path = (os.path.join(fw_path,comp))
                tskcfg = Configs.UpdateTaskConfig(fw_path)
                mcu_fw = tskcfg.getFirmwareLevel()
                mcu_fw = mcu_fw[2:]
                
                #print int(mcu_fw)
                if ( int(mcu_fw) > current_fw) :
                    self.video_up_tasks.append( (task_name,fw_path,controller,None,None) )       
            
            #FUTURE-Support: for Monitor Configurations Update
            elif (comp == DWU_COMPONENTS[1]) :
                #pass                          #do nothing when component is hw_cfg
                
                fw_path = (os.path.join(fw_path,comp))
                if os.path.exists(fw_path):
                    tskcfg = Configs.UpdateTaskConfig(fw_path)
                    mon_cfg = tskcfg.getFirmwareLevel()
                    mon_cfg = mon_cfg[2:]
                    
                    print int(mon_cfg)
                    if ( int(mon_cfg) > current_fw) :
                        self.video_up_tasks.append( (task_name,fw_path,controller,None,None) )
                else:
                    pass      
                    
            elif (comp == DWU_COMPONENTS[2]) :
                fw_path = (os.path.join(fw_path,comp))
                tskcfg = Configs.UpdateTaskConfig(fw_path)
                scalar_fw = tskcfg.getFirmwareLevel()
                scalar_fw = scalar_fw[2:]
                
                #print int(scalar_fw)
                if ( int(scalar_fw) > current_fw ) :
                    self.video_up_tasks.append( (task_name,fw_path,controller,None,None) )       
        
            elif (comp == DWU_COMPONENTS[3]) :
                fw_path = (os.path.join(fw_path,comp))
                tskcfg = Configs.UpdateTaskConfig(fw_path)
                cap_fw = tskcfg.getFirmwareLevel()
                cap_fw = cap_fw[2:]
                
                #print int(cap_fw)
                if (int(cap_fw) > current_fw) :
                    self.video_up_tasks.append( (task_name,fw_path,controller,None,None) )
                
            else:
                print 'No Firmware Update Available!'  
                TLOG.logger.info('TaskMgr::updatePendingTaskList() - No Firmware Update Available!')
                                                                                              
        except ValueError:
            print 'Value Error'                                                               
            
#==========================================================================================================
# updatePendingTaskList() - update the PendingTasks list based on the firmware we got
#==========================================================================================================    
    def updatePendingTaskList(self,task_name,fw,controller,path):
        from os import walk
        f = []
        dirpath = ''
        dirnames = ''
        filenames = ''
        
        # get a list of all the folders inside the UpdateTask Folder
        for (dirpath,dirnames,filenames) in walk(path):
            f.extend(filenames)
            break
        
        if fw.isdigit() == False:
            #for firmware levels with bcd values, we convert them to hex
            fw = int(fw,16)         
        
        # keep a record of old firmware value
        new_fw = int(fw)            
        
        #iterate thru the list of firmware folders
        folder_str  =''
        for folder in dirnames:   
            folder_name = folder
            folder=self.parseFirmwareLevel(folder)
            
            # hold the string representation of the folder name
            folder_str = folder               
         
            if folder.isdigit() == False:
                # get the hex value of the folder name for comparison
                folder = int(folder, 16)        
                
            #compare the firmware of the device to the folder name 
            #corresponding to available firmware updates            
            if int(folder) > int(new_fw):
                new_fw = folder
                
                #assign the new folder location
                new_fw_str = folder_str       
                name = folder_name
                print "Found! FW Level:", new_fw_str
                TLOG.logger.info('TaskMgr::updatePendingTaskList() - Found! FW Level:' + new_fw_str)
                
        if ( new_fw > int(fw) ):
            fw_path = (os.path.join(dirpath,name))
            if(task_name == 'SampoVideoFwFlashTask'):
                self.video_up_tasks.append( (task_name,fw_path,controller,None,None) )
            elif(task_name == 'EloTouchFwFlashTask'):
                self.touch_up_tasks.append( (task_name,fw_path,controller,None,None) )
        elif ( int(new_fw) == int(fw) ):
            print 'No Firmware Update Available!'
            TLOG.logger.info('TaskMgr::updatePendingTaskList() - No Firmware Update Available!')

#==========================================================================================================
# mergeTaskList() - Merge the Video and Touch Task Lists
#==========================================================================================================               
    def mergeTaskList(self):
        for vtask in self.video_up_tasks:
            task_type, path, controller, serial, hwid = vtask
            if task_type in (item[0] for item in self.pending_tasks) and path in (item[1] for item in self.pending_tasks):
                pass
            else:
                self.pending_tasks.append(vtask)
                
        for ttask in self.touch_up_tasks:
            task_type, path, controller, serial, hwid = ttask
            # Do not now merge DAWU Tasks - They need to be treated individually based on HWID
            if task_type in [ATMEL_FW_TASK, ATMEL_XCFG_TASK]:
                self.pending_tasks.append(ttask)
                continue
            # Yanak - we can merge as the ELO utility logic flashes All connected as appropriate
            if task_type in (item[0] for item in self.pending_tasks) and path in (item[1] for item in self.pending_tasks):
                pass
            else:
                self.pending_tasks.append(ttask)

#==========================================================================================================
# hasAvailableUpdateTasks() - Returns True if we have Update Tasks Available
#==========================================================================================================    
    def hasAvailableUpdateTasks(self):
        ret = False
               
        n = self.getMonitorList2()
        if (n != 0):
            
            for mon in self.mon_list:
                
                mon_type = mon.getMonType()
                touch_vid_pid = mon.getTouchVidPid()
                touch_fw = mon.getTouchFirmwareLevel()
                touch_serial = mon.getTouchSerialNumber()
                touch_devid = mon.getTouchDevId()
                touch_hwid = mon.getTouchHardwareId() # New for new DAWU/ELO-Panel
                
                video_vid_pid = mon.getVideoVidPid()
                video_fw = mon.getVideoFirmwareLevel()
                video_serial = mon.getVideoSerialNumber()
                video_devid  = mon.getVideoDevId()
                #video_hwid = mon.getVideoHardwareId() # Not currently used
                
                #check TouchUpdate
                self.checkForTouchFwUpdate(touch_vid_pid,touch_fw,touch_serial,mon_type,touch_devid,touch_hwid)
                
                #check VideoUpdate
                self.checkForVideoFwUpdate(video_vid_pid,video_fw,video_serial,mon_type,video_devid)
                
        else:
            TLOG.logger.info('TaskMgr::hasAvailableUpdateTasks() -No Monitors Attached.')
            print 'No Monitors Attached'
            
        self.mergeTaskList()           
        count = len(self.pending_tasks)
        if (count != 0):
            ret = True;
                    
        return ret

#==========================================================================================================
# checkRecoveryFile() - function to check if recovery trigger file exist for the Sampo Recovery task
#==========================================================================================================    
    def checkRecoveryFile(self,filename): 
        
        ret = FILE_NOT_FOUND
        
        TLOG.logger.info('TaskMgr::checkRecoveryFile------------------START---------------')
        TLOG.logger.info('TaskMgr::checkRecoveryFile-Checking Recovery file...')
        if utils.doesFileExist(filename) == True:
            ret = FILE_FOUND
            #print 'TaskMgr::checkRecoveryFile-Found Sampo Recovery trigger file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-Found Recovery file...')
        else:    
            print 'TaskMgr::checkRecoveryFile-No Recovery file...'
            TLOG.logger.info('TaskMgr::checkRecoveryFile-No Recovery file...')
                    
        TLOG.logger.info('TaskMgr::checkRecoveryFile-------------------END----------------\n')
        
        return ret
        


#==========================================================================================================
# isInSampoRecoveryMode() - Returns the sampo recovery mode status of the system
#==========================================================================================================    
    def isInSampoRecoveryMode(self):
        
        TLOG.logger.info('TaskMgr::isInSampoRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInSampoRecoveryMode-Checking if system is in sampo recovery mode...')
        recovery = False
        cmd = ''
        
        if ( self.checkRecoveryFile(AFTER_LOCAL) == FILE_FOUND ) :  
            TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Sampo Recovery file.')
            #print  'Checking Sampo Recovery file.'  
            
            recovery_file = open(AFTER_LOCAL,'r+')
            lines = recovery_file.readlines()
            for line  in lines:
                #print line
                if(RECOVERY_STR in line):                  # We look for the string '4820MonUtil', this signifies that there is a pending recovery task
            
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line)
                    print 'Found Sampo Video Recovery file.' 
                    TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Sampo Recovery Task')
                    print 'Proceed with Sampo Recovery Task'
                    
                    #ret = utils.runShell(str(line))
                    recovery = True
                    #recovery_type = 'Sampo'
                    
                    break
                
                else:
                    recovery = False
            
            recovery_file.close()
            
            self.rcv_cmd = line
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInSamopRecoveryMode()-FALSE')
                print 'Not in Sampo Recovery Mode...'
        
        return recovery

#==========================================================================================================
# isInEloRecoveryMode() - Returns the elo recovery mode status of the system
#==========================================================================================================       
    def isInEloRecoveryMode(self):   
          
        TLOG.logger.info('TaskMgr::isInEloRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInEloRecoveryMode-Checking if system is in elo recovery mode...')
        recovery = False
        cmd = ''
        
        if( self.checkRecoveryFile(ELO_RECOVERY_FILE) == FILE_FOUND ) :                  
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Elo Recovery file.')
                #print  'Checking Elo Touch Recovery file.'
               
                recovery_file = open(ELO_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                for line  in lines:
                #print line
                    if('EloRecover' in line):                  
                        
                        line = line.strip('\n')
                        tsk,cmd = line.split(' ')
                        
                        TLOG.logger.info('TaskMgr::isInRecoveryMode()- Found recovery String!- ' + line ) 
                        print 'Found Elo Touch Recovery file.'
                        TLOG.logger.info('TaskMgr::isInRecoveryMode()- Proceed with Elo Recovery Task')
                        print 'Proceed with Elo Recovery Task'
                        
                        recovery = True
                        break
                    
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInRecoveryMode - EloRecovery --------\n') 
                
                self.rcv_cmd = cmd 
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInEloRecoveryMode()-FALSE')
                print 'Not in Elo Recovery Mode...'
            
    
        return recovery
    
        
#==========================================================================================================
# isInAipRecoveryMode() - Returns the aipfwupdate recovery mode status of the system
#==========================================================================================================       
    def isInAipRecoveryMode(self):   
          
        TLOG.logger.info('TaskMgr::isInAipRecoveryMode------------------START---------------') 
        TLOG.logger.info('TaskMgr::isInAipRecoveryMode-Checking if system is in aip recovery mode...')
        recovery = False
        path = ''
        fw = ''
        
        if( self.checkRecoveryFile(AIP_RECOVERY_FILE) == FILE_FOUND ) :                  
                TLOG.logger.info('TaskMgr::isInRecoveryMode()- Checking Aip Recovery file.')
                
                recovery_file = open(AIP_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                for line  in lines:
                #print line
                    if('AIPRecover' in line):                  
                        
                        line = line.strip('\n')
                        tsk,path,fw = line.split(',')
                        
                        TLOG.logger.info('TaskMgr::isInAipRecoveryMode()- Found recovery String!- ' + line ) 
                        print 'Found Aipfwupdate Video Recovery file.'
                        TLOG.logger.info('TaskMgr::isInAipRecoveryMode()- Proceed with Aip Recovery Task')
                        print 'Proceed with Aip Recovery Task'
                        
                        recovery = True
                        break
                    
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInRecoveryMode - AipRecovery --------\n') 
                
                self.path = path
                self.fw_name = fw 
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInAipRecoveryMode()-FALSE')
                print 'Not in Aip Recovery Mode...'
            
    
        return recovery
        
#==========================================================================================================
# isInAtmelFwFlashRecoveryMode() - Did an Atmel FW Flash go bad?
#==========================================================================================================       
    def isInAtmelFwFlashRecoveryMode(self):   
	TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode------------------START---------------')
        TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode-Checking if system is in atmel recovery mode...')
        recovery = False
        fw_path = ''

        if utils.doesFileExist(utils.ATMEL_FW_RECOVERY_FILE) :
                TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode()- Checking Atmel Recovery file.')

                recovery_file = open(utils.ATMEL_FW_RECOVERY_FILE,'r+')
		lines = recovery_file.readlines()
                up_path = lines[0].strip('\n').split('=')[1]
		fw_path = lines[1].strip('\n').split('=')[1]
		serial  = lines[2].strip('\n').split('=')[1]

                TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode()- Found recovery String!fw_path- ' + fw_path)
                TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode()- Proceed with Aip Recovery Task')
	       	recovery = True
		recovery_file.close()
                TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode----------END----------\n')

                self.path = fw_path
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInAtmelFwFlashRecoveryMode()- FALSE')


        return recovery
    
        
#==========================================================================================================
# isInAtmelFwFlashRecoveryMode() - Did an XCFG Load go bad?
#==========================================================================================================       
    def isInAtmelXcfgLoadRecoveryMode(self):   
   	TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode------------------START---------------')
        TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode-Checking if system is in atmel xcfg recovery mode...')
        recovery = False
        xcfg_path = ''

        if utils.doesFileExist(utils.ATMEL_XCFG_RECOVERY_FILE) :
                TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode()- Checking Atmel xcfg Recovery file.')

                recovery_file = open(utils.ATMEL_XCFG_RECOVERY_FILE,'r+')
                lines = recovery_file.readlines()
                up_path = lines[0].strip('\n').split('=')[1]
                xcfg_path = lines[1].strip('\n').split('=')[1]
                serial  = lines[2].strip('\n').split('=')[1]

                TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode()- Found recovery String!conf_path- ' + xcfg_path)
                TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode()- Proceed with Aip Recovery Task')
                recovery = True
                recovery_file.close()
                TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode----------END----------\n')

                self.conf_path = xcfg_path
        else:
                recovery =  False
                TLOG.logger.info('TaskMgr::isInAtmelXcfgLoadRecoveryMode()- FALSE')


        return recovery
 
        
#==========================================================================================================
# performTasks() - The main function to be called by the MUS App/ Service to perform the actual update task
#==========================================================================================================    
    def performTasks(self):
        
        '''
            performTasks Sequence of Tasks
                     
            1. check if in Recovery mode (Sampo , ELO)
            2. check for available update tasks
            3. perform necessary update tasks
            
        ''' 
        
        TLOG.logger.info('TaskMgr::performTasks()=============================================================================================== S T A R T \n')       
        
        #recovery check
             
        if  self.isInEloRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doEloTouchFwFlashRecoveryTask()')
            print 'Please wait...Performing Touch Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doEloTouchFwFlashRecoveryTask(self.rcv_cmd)
            utils.runShell('reboot')
            sys.exit(0)
            
        if  self.isInAipRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doAipVideoFwFlashRecoveryTask()')
            print 'Please wait...Performing Video Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doAipVideoFwFlashRecoveryTask(self.path,self.fw_name)
            utils.runShell('reboot')
            sys.exit(0)
        
        if  self.isInAtmelFwFlashRecoveryMode() == True:
	    TLOG.logger.info('TaskMgr::performTasks()- Calling doAtmelTouchFwFlashRecoveryTask()')
            print 'Please wait...Performing Touch FW Flash Recovery, the System will reboot after the recovery operation... '
           
            self.doAtmelTouchFwFlashRecoveryTask(self.path)
            utils.runShell('reboot')
            sys.exit(0)
            
        if  self.isInAtmelXcfgLoadRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doAtmelTouchXcfgLoadRecovery()')
            print 'Please wait...Performing Touch XCFG Load Recovery, the System will reboot after the recovery operation... '
           
            self.doAtmelTouchXcfgLoadRecoveryTask(self.conf_path)
            utils.runShell('reboot')
            sys.exit(0)
            
        elif self.isInSampoRecoveryMode() == True:
            TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFwFlashRecoveryTask()')
            print 'Please wait...Performing Video Firmware Recovery, the System will reboot after the recovery operation... '
           
            self.doSampoVideoFwFlashRecoveryTask(self.rcv_cmd)
            utils.runShell('reboot')
            sys.exit(0)
                       
      
        #elif self.haveAvailableUpdateTasks() == True :
        elif self.hasAvailableUpdateTasks() == True :
            
            ###################################################################
            # Notes:
            #
            # Atmel logic has considerably changed:
            # - Multiple calls to Atmel tasks - one per chip (as needed)
            # - We don't now trust the manifest for the UP paths (human error checking)
            # - Exit on RuntimeWarning does not go into recovery.
            #
            # Previous Yanak logic and Aipfwupdate logic untouched except that the 'task' tuple
            # now has 5 members. The first 3 are used by Yanak and Aipfwupdate. The last 2 are
            # only for Atmel Touch. For Yanak/Wistron Video these will have the value 'None'
            #  
            # So what worked before should work again unless the tuple 'blows up'
            ###################################################################
            
            TLOG.logger.info('TaskMgr::performTasks()- Found Available Update tasks...')
            
            for task in self.pending_tasks:
                
                tskname, up_path, controller, serial, hwid = task
                
                TLOG.logger.info('TaskMgr::performTasks() - \nTask: ' + str(tskname) #\
                                  + '\nUP Path: ' + str(up_path) + '\nController:  ' + str(controller) #\
                                  + '\nSerial:  ' + str(serial) + '\nHardware ID:  ' + str(hwid))
                print('TaskMgr::performTasks() - \nTask: ' + str(tskname) \
                                  + '\nUP Path: ' + str(up_path) + '\nController:  ' + str(controller) \
                                  + '\nSerial:  ' + str(serial) + '\nHardware ID:  ' + str(hwid))
                
                ##########
                # Manifest #
                ##########
                
                TLOG.logger.info('TaskMgr::performTasks() -Calling MUSConfigParser() ')
                
                update_config = Configs.UpdateTaskConfig(up_path)
                           
                task_type = update_config.getTaskType()
                self.fw_path = update_config.getFirmwarePath()
                self.fw_name = update_config.getFirmwareName()
                self.controller = controller
                self.serial = serial
                self.hwid = hwid
                
                TLOG.logger.info('TaskMgr::performTasks()- TaskName= '+ task_type)
                print 'TaskMgr::performTasks()- TaskName= '+ task_type
                
                ########
                # Yanak #
                ########
                
                #4820 Monitors Video Firmware Update Tasks
                if (tskname.strip('\n') == 'SampoVideoFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doSampoVideoFwFlashTask()')
                    
                    exit_status = self.doSampoVideoFwFlashTask(task_type)          
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-SampoVideoFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        print 'Update task failed...The System will undergo reboot to recover.' 
                        utils.runShell('reboot')
                        raise RuntimeError("SampoVideoFlashTask Task Failed!")
                
                #4820 Monitors Touch Firmware Update Tasks
                elif (tskname.strip('\n') == 'EloTouchFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doEloTouchFlashTask()')
                    
                    exit_status = self.doEloTouchFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    # FLIWA_15052014   Add reboot After every firmware flash for 4820 Monitors
                    # FLIWA_15052014   Workaround when there is a 6149 Monitors around
#                    if exit_status == 0:
#                        utils.runShell('rm ' + ELO_RECOVERY_FILE)
#                        utils.runShell('reboot')
                            
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-EloTouchFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed...The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("EloTouchFwFlashTask Task Failed!")
                
                ########
                # Atmel #
                ########
                
                #
                #6149 Monitors Touch Firmware Update Tasks
                #
                elif (tskname.strip('\n') == ATMEL_FW_TASK):
                    # Extra test for manifest UP path correctness
                    test_manifest_up_path = self.fw_path.strip('\'')
                    if test_manifest_up_path != up_path:
                        print 'Error in Upgrade package: manifest UP path: ',test_manifest_up_path,' is not equal to discovered path: ',up_path,' ..ignoring update'
                        TLOG.logger.error('TaskMgr::performTasks()- Error in Upgrade package: manifest UP path: '\
                                          +test_manifest_up_path\
                                          +' is not equal to discovered path: '\
                                          +up_path\
                                          +' ..ignoring update')
                    else:
                        # Ensure task type is firmware
                        task_type = ATMEL_FW_TASK

                        try:
                            TLOG.logger.info('TaskMgr::performTasks()- Calling doAtmelTouchTask() - FW')
                            exit_status = self.doAtmelTouchTask(task_type)
                            TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                            if exit_status != 0:
                                TLOG.logger.info('TaskMgr::performTasks()- FW Update Failed!-The System will undergo reboot to recover.' )
                                #utils.updateBootCount()
                                time.sleep(1)
                                print 'FW Update task failed...The System will undergo reboot to recover.'                      
		                utils.runShell('reboot')  
                                raise RuntimeError("doAtmelTouchTask Task Failed!")
                                #
                                # Exited already
                                #
                        except RuntimeWarning:
                            #
                            # We can continue with other tasks
                            #
                            print 'doAtmelTouchTask - FW - Failed! - This task is abandoned.'
                            TLOG.logger.warn('TaskMgr::performTasks() - doAtmelTouchTask -FW - Failed - Task is abandoned')

                #
                # 6149 Monitors Touch XCFG Update Tasks
                #    
                elif (tskname.strip('\n') == ATMEL_XCFG_TASK):
                    # Extra test for manifest UP path correctness
                    test_manifest_up_path = self.fw_path.strip('\'')
                    if test_manifest_up_path != up_path:
                        print 'Error in Upgrade package: manifest UP path: ',test_manifest_up_path,' is not equal to discovered path: ',up_path,' ..ignoring update'
                        TLOG.logger.error('TaskMgr::performTasks()- Error in Upgrade package: manifest UP path: '\
                                          +test_manifest_up_path\
                                          +' is not equal to discovered path: '\
                                          +up_path\
                                          +' ..ignoring update')
                    else:
                        # Change Task type to config only (from assignment earlier)
                        task_type = ATMEL_XCFG_TASK
                        
                        try:
                            TLOG.logger.info('TaskMgr::performTasks()- Calling doAtmelTouchTask() - XCFG')
                            exit_status = self.doAtmelTouchTask(task_type)
                            TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                            if exit_status != 0:
                                TLOG.logger.info('TaskMgr::performTasks() - doAtmelTouchTask -XCFG - Failed!-The System will undergo reboot to recover.' )
                                #utils.updateBootCount()
                                time.sleep(1)
                                print 'XCFG Update task failed...The System will undergo reboot to recover.'                      
				utils.runShell('reboot')  
                                raise RuntimeError("AtmelTouchFwFlashTask Task Failed!")
                                #
                                # Exited already
                                #
                        except RuntimeWarning:
                            #
                            # We can continue with other tasks
                            #
                            print 'doAtmelTouchTask - XCFG - Failed! - This task is abandoned.'
                            TLOG.logger.warn('TaskMgr::performTasks() - doAtmelTouchTask - XCFG -  Failed - Task is abandoned')
                            
                
                
                #############
                # Aipfwupdate #
                #############
                
                #6149 Monitors Video Firmware Update Tasks
                elif (tskname.strip('\n') == 'WistronVideoFwFlashTask'):
                    TLOG.logger.info('TaskMgr::performTasks()- Calling doWistronVideoFwFlashTask()')
                    
                    exit_status = self.doWistronVideoFwFlashTask(task_type)   
                    TLOG.logger.info('TaskMgr::performTasks()-'+ ' Exit Status is:' + str(exit_status) )
                    
                    if exit_status != 0:
                        TLOG.logger.info('TaskMgr::performTasks()-WistronVideoFwFlashTask Failed!-The System will undergo reboot to recover.' )
                        #utils.updateBootCount()
                        time.sleep(1)
                        print 'Update task failed... The System will undergo reboot to recover.'                      
                        utils.runShell('reboot')  
                        raise RuntimeError("WistronVideoFwFlashTask Task Failed!")
                                                                             
        else:
                #
                # We don't have task available, so just exit from the method and end the service
                #
                TLOG.logger.info('TaskMgr::performTasks()- No Firmware Update task done.')
                print  'No Firmware Update task done.'   
            
        
        #
        # If ELO Config available always attempt it first though
        #
        if (  ( len(self.mon_list) != 0 )  and  ( utils.doesFileExist(ELO_INI) == True ) and ( self.HAS_4820_MON != False) ) :
            TLOG.logger.info('TaskMgr::performTasks()- Calling doEloConfigTask()')
            exit_status = self.doEloConfigTask(ELO_INI)
            if exit_status != 0:
                raise RuntimeError("EloConfigTask Task Failed!-exiting :" )
            #call rstelo
            self.doRstElo()
            #
        else:
            TLOG.logger.info('TaskMgr::performTasks()-EloConfigTask is not performed because either No 4820-x1x/xLx Monitor Attached or Touch Configuration file is not found.' )
            print  'EloConfigTask is not performed because either No 4820-x1x/xLx Monitor Attached or Touch Configuration file is not found.'
    
        TLOG.logger.info('TaskMgr::performTasks()================================================================================================== E N D \n')            
            
        return EXIT_SUCCESS   #will always return success with or without a task!
    
    
            
#==========================================================================================================
# doSampoVideoFwFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doSampoVideoFwFlashRecoveryTask(self,cmd):
        
        svfrt = sampo.VideoFwRecoveryFlashTask()       
        svfrt.run(cmd)     

#==========================================================================================================
# doSampoVideoFwFlashRecoveryTask() -  function to execute SampoVideoFlashRecoveryTask run method
#==========================================================================================================
    def doEloTouchFwFlashRecoveryTask(self,fw_path):
        
        etfrt = elo.TouchFwRecoveryFlashTask()       
        etfrt.run(fw_path)     
        
#==========================================================================================================
# doAipVideoFwFlashRecoveryTask() -  function to execute AipVideoFlashRecoveryTask run method
#==========================================================================================================
    def doAipVideoFwFlashRecoveryTask(self,fw_path,name):
        
        wvfrt = wistron.VideoFwRecoveryFlashTask(name)       
        wvfrt.run(fw_path)     

#==========================================================================================================
# doAtmelTouchFwFlashRecoveryTask()
#==========================================================================================================
    def doAtmelTouchFwFlashRecoveryTask(self,fw_path):
        
        rec = atmel.AtmelTouchFwFlashRecoveryTask()
        rec.run(fw_path)     

#==========================================================================================================
# doAtmelTouchXcfgLoadRecoveryTask
#==========================================================================================================
    def doAtmelTouchXcfgLoadRecoveryTask(self,conf):
        
        rec = atmel.AtmelTouchXcfgLoadRecoveryTask()
        rec.run(conf)     

#==========================================================================================================
# doSampoVideoFwFlashTask() -  function to execute SampoVideoFlashTask run method
#==========================================================================================================
    def doSampoVideoFwFlashTask(self,tsktype):
        
        result = EXIT_SUCCESS
        if (tsktype == UPDATE_TYPES[0]):
           
            svft = sampo.VideoFwFlashTask_4820_21x51x(self.fw_path,self.controller)      
            result = svft.run()      
             
        elif (tsktype == UPDATE_TYPES[1]):     
            svft = sampo.VideoFwFlashTask_4820_2Lx5Lx(self.fw_path,self.controller)      
            result = svft.run()      
        
        
        return result
    

#==========================================================================================================
# doEloTouchFwFlashTask() -  function to execute Fw Flash Command
#==========================================================================================================        
    def doEloTouchFwFlashTask(self,tsktype):
        
        exit_status = 0;
        
        if (tsktype == UPDATE_TYPES[2]):
            eyft = elo.TouchFwFlashTask_21x51x(self.fw_path,self.fw_name,self.controller)       
            exit_status  = eyft.run()    

        elif (tsktype == UPDATE_TYPES[3]):
            eaft = elo.TouchFwFlashTask_2Lx5Lx(self.fw_path,self.fw_name,self.controller)      
            exit_status = eaft.run()
             
        return exit_status


#==========================================================================================================
# doEloConfigTask() -  function to execute EloConfigTask run method
#==========================================================================================================    
    def doEloConfigTask(self,config_path):
        
        eCt = elo.ConfigTask()      
        result = eCt.run(config_path)
        return result


#==========================================================================================================
# doAtmelTouchTask() -  function to execute Atmel Touch FW Flash or XCFG Load Command
#==========================================================================================================        
    def doAtmelTouchTask(self,tsktype):
        
        exit_status = 0;
        
        if (tsktype == ATMEL_FW_TASK):
            odft = atmel.AtmelTouchFwFlashTask(self.fw_path,self.fw_name,self.controller,self.serial,self.hwid)       
            exit_status  = odft.run()
    
        elif (tsktype == ATMEL_XCFG_TASK):
            odft = atmel.AtmelTouchXcfgLoadTask(self.fw_path,self.fw_name,self.controller,self.serial,self.hwid)       
            exit_status  = odft.run()  
                  
        else:
            print 'Error: Unknown Internal Task Type', 

        return exit_status



#==========================================================================================================
# doWistronVideoFwFlashTask() -  function to execute WistronVideoFlashTask run method
#==========================================================================================================
    def doWistronVideoFwFlashTask(self,tsktype):
        
        result = EXIT_SUCCESS
        if (tsktype == UPDATE_TYPES[5]):
           
            wvft = wistron.VideoFwFlashTask_6149_DWU(self.fw_path,self.fw_name,self.controller)      
            result = wvft.run()      
            
        return result
           
#==========================================================================================================
# getDeviceState() - function to get the current state of the touch device
#==========================================================================================================
    def getDeviceState(self,device_ ):
        '''
        Checking device state
        '''
        state = UNKNOWN
        deviceFound = False
        
        TLOG.logger.info('TaskMgr::getDeviceState-----------------START---------------')
                  
        if utils.doesFileExist(ELO_DOWNLOAD) != True:
                       
            print 'TaskMgr::getDeviceState()-No ELO Download script found!'
            TLOG.logger.error('TaskMgr::getDeviceState()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        
        else:
            
            deviceFound = self.find_devices_in_fousb_mode( device_ )
            if (deviceFound == True):
                #Need to recover state of the device first:
                print  'TaskMgr::getDeviceState() :' + device_ + 'monitor needs to be recovered'
                TLOG.logger.info('TaskMgr::getDeviceState() :' + device_ + 'monitor needs to be recovered')
                state = -2
            else:
                #try to call inventory to see if any such devices are attached.        
                result = self.have_attached_display_types(device_ )
                
                if (result == NO_MONITORS_ATTACHED):
                    print  'TaskMgr::getDeviceState() :No Displays are attached at all!'
                    TLOG.logger.info('TaskMgr::getDeviceState() :No Displays are attached at all!')
                    state = -1
                    
                elif (result == True):
                    print  'TaskMgr::getDeviceState() : Has attached ' + device_ + 'display'
                    TLOG.logger.info('TaskMgr::getDeviceState() : has attached ' + device_ + 'display')
                    state = 1 
                                   
                elif (result == False):
                    print  'TaskMgr::getDeviceState() : No attached ' + device_ + 'display'
                    TLOG.logger.info('TaskMgr::getDeviceState() : No attached ' + device_ + 'display')
                    state = 2
    
        print 'TaskMgr::getDeviceState() - END '  
        TLOG.logger.info('TaskMgr::getDeviceState() -----------------END-----------------\n')
       
        return state

#==========================================================================================================
# find_devices_in_fousb_mode() - 
#==========================================================================================================
    def find_devices_in_fousb_mode(self,device_):
     
        screen = ''
        controller = ''
        result = ''
        model=''
        foundDeviceType = False
        
        find_cmd = 'elo_download -find' 
        process = subprocess.Popen(find_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE) 
        while True:
            output_line = process.stdout.readline()
            #print output_line
            if not output_line:
                break                    
            #Response line
            if "RESPONSE" in output_line:               
                res = output_line.split()
                for r in res:
                    #print r                
                    if "SCREEN" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"':
                                continue
                            screen += c
                        print screen
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :screen =' + screen)
                    elif "CONTROLLER" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if  c == '\"':
                                continue
                            controller += c
                        print controller
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :controller =' + controller)
                    elif "MODEL" in r:
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"' or c == '>' :
                                continue
                            model += c
                        print model
                        TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :model =' + model)
                    else :
                        idx = r.find("=")
                        for c in r[idx + 1:len(r)]:
                            if c == '\"':
                                continue
                            result += c
                            #print result                   
            else:
                #print 'find_devices_in_fousb_mode() - No Displays in FOUSB MODE'
                TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :- No Displays in FOUSB MODE')
    
            if device_ == model:
                print device_ + " = " + model
                foundDeviceType = True
                break
            else:
                screen=''
                controller=''
                model=''
                TLOG.logger.info('TaskMgr::find_devices_in_fousb_mode() :- No Displays in FOUSB MODE')
    
            process.wait()
    
        return foundDeviceType
         
     
#==========================================================================================================
# have_attached_display_types() - 
#==========================================================================================================
    def have_attached_display_types(self,device_ ):   
        screen = ''
        controller = ''
        result = ''
        firmware = ''
        model = '4820-'
        foundDisplayType = False
     
        elo_inventory_cmd = 'elo_download -inventory'     
        process = subprocess.Popen(elo_inventory_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        while True:
            output_line = process.stdout.readline()
            #print output_line
            if not output_line:
                break
            #Response line
            if "RESPONSE" in output_line:
                #print output_line
                if "ERROR" in output_line:
                    print  'TaskMgr::getDeviceState() - ERROR: Controller not detected'
                    TLOG.logger.error('TaskMgr::getDeviceState() ERROR: Controller not detected')
                    return NO_MONITORS_ATTACHED
                else :
                    res = output_line.split()
                    for r in res:
                        #print r
                        if "SCREEN" in r:       
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"':
                                    continue
                                screen += c
                            print screen
                            TLOG.logger.info('TaskMgr::getDeviceState() :screen =' + screen)
                        elif "CONTROLLER" in r:
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if  c == '\"':
                                    continue
                                controller += c
                            #print controller
                            TLOG.logger.info('TaskMgr::getDeviceState() :controller =' + controller)
                        #elif "MODEL" in r:
                        elif "4820" in r:
                            idx = r.find("-")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"' or c == '>' :
                                    continue
                                model += c
                            #print model
                            TLOG.logger.info('TaskMgr::getDeviceState() :model =' + model)
                        else :
                            idx = r.find("=")
                            for c in r[idx + 1:len(r)]:
                                if c == '\"':
                                    continue
                                result += c
                                #print result
            elif "DATA" in output_line:
                idx = output_line.find(">")
                for c in output_line[idx + 1:len(output_line)]:
                    if c == '<':
                        break
                    firmware += c
                #print firmware 
                TLOG.logger.info('TaskMgr::getDeviceState() :firmware =' + firmware)
                
            if device_ == model:
                print device_ + " = " + model
                foundDisplayType = True                  
                break
            else:
                screen = ''
                controller = ''
                result = ''
                firmware = ''
                model = '4820-'
            
            process.wait()

        return foundDisplayType 
    

#==========================================================================================================
'''
TouchMonitor - abstraction of the touch monitor 
'''
#==========================================================================================================

class TouchMonitor(object):
    def __init__(self):
        '''
        Constructor
        '''
               
        #TLOG.logger.info('TouchMonitor Constructed')
             
        self.mon_type = ''
        self.screenNumber = ''
    
        #touch components
        self.touch_serial = ''
        self.touchFirmwareLevel = ''
        self.touch_vidpid = ''
        self.touch_dev_id = ''
        self.touch_hwid = ''
    
        #video components
        self.video_serial =''        
        self.videoFirmwareLevel = ''
        self.video_vidpid = ''
        self.video_dev_id = ''
        self.video_hwid = ''
     

    def setMonType(self,montype):
        self.mon_type = montype
    
    def setScreenNo(self,scrn_no):
        self.screenNumber = scrn_no

    
    def setVideoSerialNumber(self,serial):
        self.video_serial = serial
    
    def setVideoVidPid(self,vidpid):
        self.video_vidpid = vidpid

    def setVideoDevId(self,dev_id):
        self.video_dev_id = dev_id
        
    def setVideoFwLevel(self,fwlevel):
        self.videoFirmwareLevel = fwlevel
        
    def setVideoHardwareId(self,hwid):
        self.video_hwid = hwid
    
    
    def setTouchSerialNumber(self,serial):
        self.touch_serial = serial

    def setTouchVidPid(self,vidpid):
        self.touch_vidpid = vidpid
        
    def setTouchDevId(self,dev_id):
        self.touch_dev_id = dev_id
    
    def setTouchFwLevel(self,fwlevel):
        self.touchFirmwareLevel = fwlevel

    def setTouchHardwareId(self,hwid):
        self.touch_hwid = hwid
        
    
    def getMonType(self):
        
        TLOG.logger.info('TouchMonitor::getMonType() - monitor type = ' + self.mon_type )
        
        return self.mon_type
                     
    def getScreenNumber(self):
        
        TLOG.logger.info('TouchMonitor::getScreenNumber() - screen/hub number = ' + self.screenNumber )
        
        return self.screenNumber


    def getTouchVidPid(self):
        
        TLOG.logger.info('TouchMonitor::getTouchVidPid() - vidpid = ' + self.touch_vidpid )
        
        return self.touch_vidpid

    def getVideoVidPid(self):
        
        TLOG.logger.info('TouchMonitor::getVideoVidPid() - vidpid = ' + self.video_vidpid  )
        
        return self.video_vidpid
    

    def getTouchDevId(self):
        
        TLOG.logger.info('TouchMonitor::getTouchDevId() - dev id = ' + self.touch_dev_id )
        
        return self.touch_dev_id

    def getVideoDevId(self):
        
        TLOG.logger.info('TouchMonitor::getVideoDevId() - dev id = ' + self.video_dev_id  )
        
        return self.video_dev_id

    
    def getVideoSerialNumber(self):
        
        TLOG.logger.info('TouchMonitor::getControllerId() - controller id = ' + self.video_serial )
        
        return self.video_serial

    def getTouchSerialNumber(self):
        
        TLOG.logger.info('TouchMonitor::getControllerId() - controller id = ' + self.touch_serial )
        
        return self.touch_serial   

    
    def getTouchFirmwareLevel(self):
        
        TLOG.logger.info('TouchMonitor::getFirmwareLevel() - fw_level = ' + self.touchFirmwareLevel  )
        
        return self.touchFirmwareLevel 
    
    def getVideoFirmwareLevel(self):
        
        TLOG.logger.info('TouchMonitor::getFirmwareLevel() - fw_level = ' + self.videoFirmwareLevel  )
        
        return self.videoFirmwareLevel   
    

    def getTouchHardwareId(self):
        
        TLOG.logger.info('TouchMonitor::getTouchHardwareId() - fw_level = ' + self.touch_hwid )
        
        return self.touch_hwid
    
    def getVideoHardwareId(self):
        
        TLOG.logger.info('TouchMonitor::getVideoHardwareId() - fw_level = ' + self.video_hwid  )
        
        return self.video_hwid


    def printInfo(self):
        print 'screen# = ' + self.screenNumber

        print 'touch serial# =' + self.touch_serial    
        print 'touch controllerId =' + self.touch_vidpid
        print 'touch firmware = ' + self.touchFirmwareLevel
        print 'touch hardware id = ' + self.touch_hwid
    
        print 'video serial# =' + self.video_serial    
        print 'video controllerId =' + self.video_vidpid      
        print 'video firmware = ' + self.videoFirmwareLevel
        print 'video hardware id = ' + self.video_hwid


